<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text"/>

<xsl:include href="util.xslt"/>

<xsl:template name="build-varname">
	<xsl:variable name="form-name" select="ancestor::form/@name"/>
	<xsl:variable name="input-name" select="@name"/>
	<xsl:variable name="input-value" select="@value"/>

	<xsl:value-of select="$form-name"/>
	<xsl:text>_</xsl:text>
	<xsl:value-of select="$input-name"/>
	<xsl:if test="string-length($input-value) > 0">
		<xsl:text>_</xsl:text>
		<xsl:value-of select="$input-value"/>
	</xsl:if>
</xsl:template>

<xsl:template name="_build-enabled-by">
	<xsl:param name="list"/>
	<xsl:variable name="token">
		<xsl:choose>
			<xsl:when test="contains($list, ',')">
				<xsl:value-of select="substring-before($list, ',') or string($list)"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$list"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:variable>
	<xsl:if test="contains($list, ',')">
		<xsl:call-template name="_build-enabled-by">
			<xsl:with-param name="list" select="substring-after($list, ',')"/>
		</xsl:call-template>
		<xsl:text> || </xsl:text>
	</xsl:if>
	<xsl:choose>
		<xsl:when test="contains($token, '[')">
			<xsl:variable name="index" select="substring-before(substring-after($token, '['), ']')"/>
			<!-- Find the name of the radio button that matches our index -->
			<xsl:variable name="name" select="//input[@type='radio' and @value=$index]/@name"/>

			<xsl:value-of select="substring-before($token, '[')"/>
			<xsl:text>[</xsl:text>
			<xsl:for-each select="//input[@type='radio' and @name=$name]">
				<xsl:if test="@value = $index">
					<xsl:value-of select="position() - 1"/>
				</xsl:if>
			</xsl:for-each>
			<xsl:text>]</xsl:text>
			<xsl:text>.checked == 1</xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$token"/>
			<xsl:text>.checked == 1</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="build-enabled-by">

	<xsl:param name="type"/>
	<xsl:param name="enabled-by"/>

	<xsl:variable name="value" select="@value"/>

	<xsl:for-each select="//*[@enabled-by]">
		<xsl:if test="string-length(@enabled-by) > 0">
			<xsl:text>printf("&lt;script type=\"text/javascript\"&gt;</xsl:text>
			<xsl:text>function document_</xsl:text>
			<xsl:value-of select="ancestor::form/@name"/>
			<xsl:text>_</xsl:text>
			<xsl:value-of select="@name"/>	
			<xsl:text>_check_dependencies() {if (</xsl:text>
			<xsl:call-template name="_build-enabled-by">
				<xsl:with-param name="list" select="@enabled-by"/>
			</xsl:call-template>
			<xsl:text>) {</xsl:text>
			<xsl:text>document.</xsl:text>
			<xsl:value-of select="ancestor::form/@name"/>
			<xsl:text>.</xsl:text>
			<xsl:value-of select="@name"/>
			<xsl:text>.disabled = false;} else {</xsl:text>
			<xsl:text>document.</xsl:text>
                        <xsl:value-of select="ancestor::form/@name"/>
                        <xsl:text>.</xsl:text>
                        <xsl:value-of select="@name"/>
                        <xsl:text>.disabled = true;}}</xsl:text>
			<xsl:text>&lt;/script&gt;");</xsl:text>
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template name="build-onclick">

        <xsl:param name="onclick"/>

	<xsl:variable name="name" select="@name"/>

	<xsl:text>onclick=\"</xsl:text>

	<xsl:for-each select="//*[@enabled-by]">
		<xsl:if test="contains(@enabled-by,$name)">
			<xsl:text>document_</xsl:text>
			<xsl:value-of select="ancestor::form/@name"/>
			<xsl:text>_</xsl:text>
			<xsl:value-of select="@name"/>	
			<xsl:text>_check_dependencies();</xsl:text>
		</xsl:if>
	</xsl:for-each>

        <xsl:if test="string-length($onclick) > 0">
                <xsl:value-of select="$onclick"/>
                <xsl:text>();</xsl:text>
        </xsl:if>

	<xsl:text>\"</xsl:text>
</xsl:template>

<xsl:template match="webapp">
	<xsl:text>/* WARNING: This C source code was generated from XML: DO NOT EDIT BY HAND! */</xsl:text>
	<xsl:value-of select="$cr"/>
	<xsl:value-of select="$cr"/>
	<xsl:text>
#include &lt;stdio.h&gt;
#include &lt;stdio.h&gt;
#include &lt;string.h&gt;
#include &lt;stdlib.h&gt;
#include &lt;unistd.h&gt;
#include &lt;sys/types.h&gt;
#include &lt;sys/stat.h&gt;
#include &lt;cgi-util.h&gt;
</xsl:text>
	<xsl:apply-templates select="(fieldset|form)" mode="pound-statements"/>
	<xsl:value-of select="$cr"/>
	<xsl:text>
void print_form (char *sidebar)
{
	struct stat buf;</xsl:text>
	<xsl:value-of select="$cr"/>
	<xsl:apply-templates select="(//input|//select)" mode="declare-vars"/>
	<xsl:text>printf("</xsl:text>
	<xsl:apply-templates select="top"/>
	<xsl:text>", sidebar);
</xsl:text>
	<xsl:call-template name="build-enabled-by"/>
	<xsl:apply-templates select="(fieldset|form)" mode="print-form"/>
	<xsl:text>printf("</xsl:text>
	<xsl:apply-templates select="bottom"/>
	<xsl:text>");
}

void save_data(void)
{
	FILE *outfile;
	struct stat buf;

	char *mode;
	</xsl:text>
	<xsl:apply-templates select="(fieldset/form/input|form/input|fieldset/form/select|form/select)" mode="declare-vars"/>
	<xsl:text>
	if ((mode = cgi_getentrystr("mode")) == NULL
            || *mode == 0x00) {
		cgi_error ("Error: missing mode");
        }
	</xsl:text>
	<xsl:apply-templates select="(fieldset|form)" mode="save-input"/>
	<xsl:text>
}

char *get_service_name(void)
{
        return g_strdup("</xsl:text><xsl:value-of select="@short-name"/><xsl:text>");
}</xsl:text>
</xsl:template>

<xsl:template match="fieldset" mode="pound-statements">
	<xsl:apply-templates select="form" mode="pound-statements"/>
</xsl:template>

<xsl:template match="legend" mode="print-form">
	<xsl:text>printf("&lt;legend&gt;");</xsl:text>
	<xsl:choose>
		<xsl:when test="count(child::*) > 0">
			<xsl:apply-templates select="*" mode="output-no-stored-settings"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text>printf("</xsl:text>
			<xsl:value-of select="."/>
			<xsl:text>");</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:text>printf("&lt;/legend&gt;");</xsl:text>
</xsl:template>

<xsl:template match="legend" mode="output-no-stored-settings">
	<xsl:text>printf("&lt;legend&gt;");</xsl:text>
	<xsl:choose>
		<xsl:when test="count(child::*) > 0">
			<xsl:apply-templates select="*" mode="output-no-stored-settings"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text>printf("</xsl:text>
			<xsl:value-of select="."/>
			<xsl:text>");</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:text>printf("&lt;/legend&gt;");</xsl:text>
</xsl:template>

<xsl:template match="legend" mode="output-stored-settings">
	<xsl:text>printf("&lt;legend&gt;");</xsl:text>
	<xsl:choose>
		<xsl:when test="count(child::*) > 0">
			<xsl:apply-templates select="*" mode="output-no-stored-settings"/>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text>printf("</xsl:text>
			<xsl:value-of select="."/>
			<xsl:text>");</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:text>printf("&lt;/legend&gt;");</xsl:text>
</xsl:template>

<xsl:template match="fieldset" mode="print-form">
	<xsl:text>printf("&lt;fieldset&gt;");</xsl:text>
	<xsl:apply-templates select="legend" mode="print-form"/>
	<xsl:apply-templates select="form|input|select|fieldset|br" mode="print-form"/>
	<xsl:text>printf("&lt;/fieldset&gt;");</xsl:text>
</xsl:template>

<xsl:template match="fieldset" mode="output-no-stored-settings">
	<xsl:text>printf("&lt;fieldset&gt;");</xsl:text>
	<xsl:apply-templates select="legend" mode="output-no-stored-settings"/>
	<xsl:apply-templates select="form|input|select|fieldset|br" mode="output-no-stored-settings"/>
	<xsl:text>printf("&lt;/fieldset&gt;");</xsl:text>
</xsl:template>

<xsl:template match="fieldset" mode="output-stored-settings">
	<xsl:text>printf("&lt;fieldset&gt;");</xsl:text>
	<xsl:apply-templates select="legend" mode="output-stored-settings"/>
	<xsl:apply-templates select="form|input|select|fieldset|br" mode="output-stored-settings"/>
	<xsl:text>printf("&lt;/fieldset&gt;");</xsl:text>
</xsl:template>

<xsl:template match="fieldset" mode="save-input">
	<xsl:apply-templates select="form" mode="save-input"/>
</xsl:template>

<xsl:template match="fieldset" mode="read-stored-settings">
</xsl:template>

<xsl:template match="form" mode="pound-statements">
	<xsl:text>#define OUTFILE_</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text> "</xsl:text>
	<xsl:value-of select="@save-to-dir"/>
	<xsl:value-of select="@name"/>
	<xsl:text>"
</xsl:text>
</xsl:template>

<xsl:template match="form" mode="print-form">
	<xsl:text>
	printf("&lt;p&gt;&lt;form name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\" action=\"</xsl:text>
	<xsl:value-of select="/webapp/@name"/>
	<xsl:text>\" method=\"post\"&gt;");
	if (stat (OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, &amp;buf) == -1) {</xsl:text>
		<xsl:apply-templates select="*" mode="output-no-stored-settings"/>
	<xsl:text>
	} else { /* FIXME: need conf parser lib. Unfortunately, glib requires [GROUPS] */
		FILE *outfile;

		char buf[BUFSIZ];
		if ((outfile = fopen (OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, "r")) == NULL) {
			cgi_error ("Error opening " OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
		}

		while (fgets (buf, sizeof (buf), outfile) != NULL) {
			buf[strlen (buf) - 1] = 0x00; /* get rid of '\n' */

	</xsl:text>
	<xsl:apply-templates select="*" mode="read-stored-settings"/>
	<xsl:text>
		}
	</xsl:text>
	<xsl:apply-templates select="*" mode="output-stored-settings"/>
	<xsl:text>
		if (fclose (outfile) == EOF) {
			cgi_error ("Error closing " OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
		}
	}
	</xsl:text>
	<xsl:apply-templates select="submit"/>
	<xsl:text>
	printf("&lt;input name=\"mode\" type=\"hidden\" value=\"</xsl:text><xsl:value-of select="@name"/><xsl:text>\"/&gt;\n");
	printf("&lt;/form&gt;&lt;p&gt;\n");
	</xsl:text>
</xsl:template>

<xsl:template match="form" mode="save-input">
	<xsl:text>

	if (g_ascii_strcasecmp(mode, "</xsl:text><xsl:value-of select="@name"/><xsl:text>") == 0) {

	if (stat (OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, &amp;buf) == 0) {
		/* Deleted by appliance-configd after running all scripts */
		if (rename (OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text> ".old") == -1) {
			cgi_error ("Error renaming old configuration");
		}
	}
	if ((outfile = fopen (OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, "w")) == NULL) {
		cgi_error ("Error opening " OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
	}
	</xsl:text>
	<xsl:choose>
		<xsl:when test="@save-data-as = 'config'"> 
			<xsl:text>
			fprintf (outfile, "# Created by appliance-config, do not edit by hand\n");
			fprintf (outfile, "\n");
			</xsl:text>
			<xsl:apply-templates select="(input|select)" mode="save-input-to-config"/>
		</xsl:when>
		<xsl:when test="@save-data-as = 'spool'"> 
			<xsl:apply-templates select="(input|select)" mode="save-input-to-spool"/>
			<xsl:text>printf("\n");</xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:text>cgi_error ("Error: Invalid save-data-as value:</xsl:text><xsl:value-of select="@save-data-as"/><xsl:text>");</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
	<xsl:text>
	if (fclose (outfile) == EOF) {
		cgi_error ("Error closing " OUTFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
	}
	}
	</xsl:text>
</xsl:template>

<xsl:template match="input|select" mode="declare-vars">
	<xsl:text>	char *</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text> = NULL;</xsl:text>
</xsl:template>

<xsl:template match="input|select" mode="save-input-to-config">
	<xsl:text>
	if ((</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text> = cgi_getentrystr ("</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>")) == NULL || *</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text> == 0x00) {
		</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text> = "off";
	}
	fprintf (outfile, "</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text>=%s\n", </xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text>);
	</xsl:text>
</xsl:template>

<xsl:template match="input|select" mode="save-input-to-spool">
	<xsl:text>
	if ((</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text> = cgi_getentrystr ("</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>")) == NULL || *</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text> == 0x00) {
		</xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text> = "off";
	}
	fprintf (outfile, "%s ", </xsl:text>
	<xsl:call-template name="build-varname"/>
	<xsl:text>);
	</xsl:text>
</xsl:template>

<xsl:template match="input" mode="output-no-stored-settings">
	<xsl:text>
		printf("</xsl:text>
	<xsl:value-of select="@label"/>
	<xsl:text>&lt;input name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\" type=\"</xsl:text>
	<xsl:value-of select="@type"/>
	<xsl:text>\" </xsl:text>
	<xsl:call-template name="build-onclick">
		<xsl:with-param name="onclick" select="@onclick"/>
	</xsl:call-template>
	<xsl:text>/&gt;");</xsl:text>
</xsl:template>

<xsl:template match="input" mode="read-stored-settings">
	<xsl:text>
			if (g_ascii_strncasecmp (buf, "</xsl:text>

	<xsl:call-template name="build-varname"/>
	<xsl:text>=", strlen ("</xsl:text>

	<xsl:call-template name="build-varname"/>

	<xsl:text>=")) == 0) {</xsl:text>
	<xsl:value-of select="$cr"/>
	<xsl:value-of select="$tab"/>
	<xsl:value-of select="$tab"/>
	<xsl:value-of select="$tab"/>
	<xsl:value-of select="$tab"/>

	<xsl:call-template name="build-varname"/>

	<xsl:text> = g_strdup (buf + strlen ("</xsl:text>

	<xsl:call-template name="build-varname"/>

	<xsl:text>="));
			}</xsl:text>
</xsl:template>

<xsl:template match="input" mode="output-stored-settings">
	<xsl:text>
			if (g_ascii_strcasecmp ("on", </xsl:text>

	<xsl:call-template name="build-varname"/>

	<xsl:text>) == 0) {
				printf("</xsl:text>
		<xsl:value-of select="@label"/>
		<xsl:text>&lt;input name=\"</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>\" type=\"</xsl:text>
		<xsl:value-of select="@type"/>
		<xsl:text>\" </xsl:text>
		<xsl:call-template name="build-onclick">
			<xsl:with-param name="onclick" select="@onclick"/>
		</xsl:call-template>
		<xsl:text> checked=\"checked\" /&gt;");
			} else {
				printf("</xsl:text>
		<xsl:value-of select="@label"/>
		<xsl:text>&lt;input name=\"</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>\" type=\"</xsl:text>
		<xsl:value-of select="@type"/>
		<xsl:text>\" </xsl:text>
		<xsl:call-template name="build-onclick">
			<xsl:with-param name="onclick" select="@onclick"/>
		</xsl:call-template>
		<xsl:text>/&gt;");
			}</xsl:text>
</xsl:template>

<xsl:template match="select" mode="output-no-stored-settings">
	<xsl:text>
		printf("</xsl:text>
	<xsl:value-of select="@label"/>
	<xsl:text>&lt;select name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\"&gt;");</xsl:text>
	<xsl:apply-templates select="*" mode="output-no-stored-settings"/>
	<xsl:text>
		printf("&lt;/select&gt;");
	</xsl:text>
</xsl:template>

<xsl:template match="select" mode="output-stored-settings">
	<xsl:text>
		printf("</xsl:text>
	<xsl:value-of select="@label"/>
	<xsl:text>&lt;select name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\"&gt;");</xsl:text>
	<xsl:apply-templates select="*" mode="output-stored-settings"/>
	<xsl:text>
		printf("&lt;/select&gt;");
	</xsl:text>
</xsl:template>

<xsl:template match="select" mode="read-stored-settings">
</xsl:template>

<xsl:template match="option" mode="output-no-stored-settings">
	<xsl:text>
		printf("&lt;option value=\"</xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text>\"&gt;</xsl:text>
	<xsl:value-of select="."/>
	<xsl:text>&lt;/option&gt;");</xsl:text>
</xsl:template>

<xsl:template match="option" mode="output-stored-settings">
	<xsl:text>
		printf("&lt;option value=\"</xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text>\"&gt;</xsl:text>
	<xsl:value-of select="."/>
	<xsl:text>&lt;/option&gt;");</xsl:text>
</xsl:template>

<xsl:template match="call-print-options-fn" mode="output-no-stored-settings">
	<xsl:text>print_document_</xsl:text>
	<xsl:value-of select="ancestor::form/@name"/>
	<xsl:text>_</xsl:text>
	<xsl:value-of select="ancestor::form/@name"/>
	<xsl:text>_options ("document.</xsl:text>
	<xsl:value-of select="../../@name"/>
	<xsl:text>.</xsl:text>
	<xsl:value-of select="ancestor::form/@name"/>
	<xsl:text>");</xsl:text>
</xsl:template>

<xsl:template match="call-print-options-fn" mode="output-stored-settings">
	<xsl:text>print_document_</xsl:text>
	<xsl:value-of select="ancestor::form/@name"/>
	<xsl:text>_</xsl:text>
	<xsl:value-of select="ancestor::form/@name"/>
	<xsl:text>_options ("document.</xsl:text>
	<xsl:value-of select="ancestor::form/@name"/>
	<xsl:text>.</xsl:text>
	<xsl:value-of select="ancestor::form/@name"/>
	<xsl:text>");</xsl:text>
</xsl:template>

<xsl:template match="br" mode="output-no-stored-settings">
	<xsl:text>printf("&lt;br/&gt;");</xsl:text>
</xsl:template>

<xsl:template match="br" mode="output-stored-settings">
	<xsl:text>printf("&lt;br/&gt;");</xsl:text>
</xsl:template>

<xsl:template match="submit">
	<xsl:text>
	printf("&lt;p&gt;&lt;input name=\"submit\" type=\"submit\" value=\"</xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text>\"/&gt;&lt;p&gt;");</xsl:text>
	<xsl:value-of select="$cr"/>
</xsl:template>

</xsl:stylesheet>
